import re

# 7.1 Slug Generator
def slugify(text: str) -> str:
    return re.sub(r'[^a-zA-Z0-9]+', '-', text.lower()).strip('-')

# 7.4 Regex Utility Collection
COMMON_PATTERNS = {
    "email": r"^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$",
    "url": r"^(https?|ftp)://[^\s/$.?#].[^\s]*$",
}

# 7.5 Levenshtein Distance Helper
def levenshtein_distance(s1: str, s2: str) -> int:
    if len(s1) < len(s2):
        return levenshtein_distance(s2, s1)
    if len(s2) == 0:
        return len(s1)
    previous_row = range(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            insertions = previous_row[j + 1] + 1
            deletions = current_row[j] + 1
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row
    return previous_row[-1]